/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2016-2018 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "externalFunction1Force.H"
#include "rigidBodyModel.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace RBD
{
namespace restraints
{
    defineTypeNameAndDebug(externalFunction1Force, 0);

    addToRunTimeSelectionTable
    (
        restraint,
        externalFunction1Force,
        dictionary
    );
}
}
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::RBD::restraints::externalFunction1Force::externalFunction1Force
(
    const word& name,
    const dictionary& dict,
    const rigidBodyModel& model
)
:
    restraint(name, dict, model)
{
    read(dict);
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::RBD::restraints::externalFunction1Force::~externalFunction1Force()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

void Foam::RBD::restraints::externalFunction1Force::restrain
(
    scalarField& tau,
    Field<spatialVector>& fx
) const
{
    // At the moment, time information is not available.
    // Thus, constant value is considered.
    // It might be implemented with TimeFunction1 later
    const vector force = externalForce_().value(0);
    const vector moment(location_ ^ force);

    if (model_.debug)
    {
        Info<< " location " << location_ << endl;
    }

    // Accumulate the force for the restrained body
    fx[bodyIndex_] += spatialVector(moment, force);
}


bool Foam::RBD::restraints::externalFunction1Force::read
(
    const dictionary& dict
)
{
    restraint::read(dict);


    coeffs_.lookup("location") >> location_;
    const word& type = coeffs_.lookup("Function1Type");
    const dictionary& subDict = coeffs_.subDict(type+"Coeffs");
    externalForce_ = Function1<vector>::New
    (
        type,
        subDict
    );

    return true;
}


void Foam::RBD::restraints::externalFunction1Force::write
(
    Ostream& os
) const
{
    restraint::write(os);
    const word subDictName = externalForce_().name() + "Coeffs";

    os.writeKeyword("location")
        << location_ << token::END_STATEMENT << nl;
    os.writeKeyword("Function1Type")
        << externalForce_().name() << token::END_STATEMENT << nl;
    os.writeKeyword(subDictName) << nl;
    os << token::BEGIN_BLOCK << incrIndent << nl;
    os << externalForce_() << nl;
    os << decrIndent << token::END_BLOCK << endl;

}


// ************************************************************************* //
